import os
import sys
import json


from log import logger, get_rotating_logger


class LogTweetHandler(object):

    def __init__(self, logfile=None, interval=60, when="M"):
        """ interval is number of minutes for each log file """
        if logfile:
            self.logger = get_rotating_logger(logfile, interval)
            self.logfile = logfile

    def _validate(self, key, record):
        if key in record and record[key]:
            return True
        return False
   
    def validate_geotweet(self, record):
        """ check that stream record is actual tweet with coordinates """
        return self._validate('user', record) and self._validate('geo', record)

    def handle(self, record):
        """ called when a record is received from the Twitter API stream """
        if self.validate_geotweet(record):
            tweet = Tweet(record)
            self.logger.info(tweet.as_json())
           

class Tweet(object):

    def __init__(self, record):
        self.extract_user(record['user'])
        self.extract_meta(record)
        self.record = record

    def extract_user(self, user):
        self.user_id = user['id']
        self.name = user['name']
        self.screen_name = user['screen_name']
        self.description = user['description']
        self.location = user['location']
        self.friends_count = user['friends_count']
        self.followers_count = user['followers_count']

    def extract_meta(self, tweet):
        self.tweet_id = tweet['id_str']
        self.source = tweet['source']
        self.created_at = tweet['created_at']
        self.timestamp = tweet['timestamp_ms']
        self.lonlat = tweet['coordinates']['coordinates']
  
    def as_json(self):
        return json.dumps(dict(
            user_id=self.user_id, name=self.name, screen_name=self.screen_name,
            description=self.description, location=self.location,
            friends_count=self.friends_count, followers_count=self.followers_count,
            tweet_id=self.tweet_id, source=self.source, created_at=self.created_at,
            timestamp=self.timestamp, lonlat=self.lonlat,
            text=self.record['text']
        ))

    def display(self):
        print self.user_id
        print self.name
        print self.screen_name
        print self.description
        print self.location
        print "TWEET"
        print self.created_at, self.timestamp
        print "GEO", self.lonlat
        print self.source
        print self.record['text']



import os
import sys
import pyinotify

root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(root)

from geotweet.log import logger
from geotweet.load import S3Loader


class LogListener(object):
    
    def __init__(self, args):
        """
        Listen for changes to files in log_dir

        When TwitterStream rotates a log the event handler will be called
        The LogEventS3Handler will load the log file into a s3 bucket on AWS
        """
        self.log_dir = args.log_dir
        wm = pyinotify.WatchManager()
        handler = LogEventHandler(bucket=args.bucket, region=args.region)
        self.notifier = pyinotify.Notifier(wm, handler)
        wm.add_watch(self.log_dir, pyinotify.ALL_EVENTS)

    def start(self):
        msg = "Start listening for events in directory: {0}"
        logger.info(msg.format(self.log_dir))
        self.notifier.loop()


class LogEventHandler(pyinotify.ProcessEvent):
    
    def my_init(self, bucket=None, region=None):
        """
        This is automatically called from ProcessEvent.__init__()
        """
        self.bucket = bucket
        self.s3 = S3Loader(bucket, region)
        try:
            self.s3.valid()
        except EnvironmentError as e:
            logger.error(e)
            sys.exit(1)

    def process_IN_MOVED_TO(self, event):
        """
        Log file was rotated. Send to s3
        """
        logger.debug("MOVED LOG {0} {1}".format(event.maskname, event.pathname))
        if self.s3:
            log = "Loading to S3 bucket {0}: {1}".format(self.bucket, event.pathname)
            logger.info(log)
            self.s3.store(event.pathname)
            logger.info("Finished loading: {0}".format(event.pathname))



import os
import sys
import time

import twitter

from log import logger


class TwitterClient(object):
    
    def __init__(self):
        try:
            self.api = twitter.Api(
                consumer_key=os.getenv('TWITTER_CONSUMER_KEY'),
                consumer_secret=os.getenv('TWITTER_CONSUMER_SECRET'),
                access_token_key=os.getenv('TWITTER_ACCESS_TOKEN_KEY'),
                access_token_secret=os.getenv('TWITTER_ACCESS_TOKEN_SECRET')
            )
            self.api.VerifyCredentials()
        except twitter.error.TwitterError as e:
            logger.error("Error connecting to twitter API " + str(e))
            sys.exit(1)


class TwitterStream(object):
    
    ENVVARS = [
        "TWITTER_CONSUMER_KEY",
        "TWITTER_CONSUMER_SECRET",
        "TWITTER_ACCESS_TOKEN_KEY",
        "TWITTER_ACCESS_TOKEN_SECRET"
    ]

    def __init__(self):
        self.validate_envvar()                        
        self.api = TwitterClient().api

    def validate_envvar(self):
        error = "Error: Make sure following environment variables are set\n"
        error += "\t" + "\n\t".join(self.ENVVARS) + "\n"
        for env in self.ENVVARS:
            if not os.getenv(env, None):
                value = "environment variable {0} not set".format(env)
                logger.error(value)
                raise EnvironmentError(value)
    
    def stream(self, streamer, handler, timeout):
        if timeout:
            stop = time.time() + timeout
        for record in streamer:
            if timeout and time.time() > stop:
                break
            handler.handle(record)

    def start(self, handler, timeout=0, locations=None):
        if locations:
            streamer = self.api.GetStreamFilter(locations=locations)
        else:
            streamer = self.api.GetStreamSample()
        self.stream(streamer, handler, timeout)



import logging
from logging.handlers import TimedRotatingFileHandler
import os
import sys


LOG_NAME = 'geotweet'
LOG_FILE = os.getenv('GEOTWEET_LOG', '/tmp/geotweet.log')
LOG_LEVEL = logging.DEBUG
TWITTER_LOG_NAME = 'twitter-stream'


def get_logger():
    logformat = '%(levelname)s %(pathname)s %(asctime)s: %(message)s'
    formatter = logging.Formatter(logformat)
    logger = logging.getLogger(LOG_NAME)
    logger.setLevel(LOG_LEVEL)
    # output file
    fh = logging.FileHandler(LOG_FILE)
    fh.setLevel(LOG_LEVEL)
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    # standard out
    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(LOG_LEVEL)
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    return logger


def get_rotating_logger(logfile, interval, when="M"):
    logger = logging.getLogger(TWITTER_LOG_NAME)
    logger.setLevel(LOG_LEVEL)
    handler = TimedRotatingFileHandler(logfile, when=when, interval=interval)
    logger.addHandler(handler)
    return logger


logger = get_logger()



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


setup(
    name='geotweet',
    version='0.1.1',
    description='Fetch geographic tweets from Twitter Streaming API',
    author='Jeffrey Alan Meyers',
    author_email='jeffrey.alan.meyers@gmail.com',
    url='https://github.com/meyersj/geotweet',
    packages=['geotweet'],
    scripts=['bin/geotweet'],
    install_requires=[
        'argparse',
        'boto3',
        'python-twitter',
        'pyinotify',
    ]
)
